@@ -48,6 +48,16 @@ static gboolean spi_atk_bridge_property_event_listener (GSignalInvocationHint *s
 							guint                  n_param_values,
 							const GValue          *param_values,
 							gpointer               data);
+static gboolean
+spi_atk_bridge_window_event_listener (GSignalInvocationHint *signal_hint,
+				guint n_param_values,
+				const GValue *param_values,
+				gpointer data);
+static gboolean
+spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
+				     guint n_param_values,
+				     const GValue *param_values,
+				     gpointer data);
 static gboolean spi_atk_bridge_signal_listener         (GSignalInvocationHint *signal_hint,
 							guint                  n_param_values,
 							const GValue          *param_values,
@@ -100,7 +110,7 @@ atk_bridge_init (gint *argc, gchar **argv[])
   CORBA_exception_init(&ev);
 
   registry = bonobo_activation_activate_from_id (
-	  "OAFIID:Accessibility_Registry:proto0.1", 0, NULL, &ev);
+	  "OAFIID:Accessibility_Registry:1.0", 0, NULL, &ev);
   
   if (ev._major != CORBA_NO_EXCEPTION)
     {
@@ -183,6 +193,30 @@ spi_atk_register_event_listeners (void)
   id = atk_add_global_event_listener (spi_atk_bridge_property_event_listener,
 				      "Gtk:AtkObject:property-change");
   g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:create");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:destroy");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:minimize");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:maximize");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:restore");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:activate");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_window_event_listener,
+				      "window:deactivate");
+  g_array_append_val (listener_ids, id);
+  id = atk_add_global_event_listener (spi_atk_bridge_state_event_listener,
+				      "Gtk:AtkObject:state-change");
+  g_array_append_val (listener_ids, id);
 
   add_signal_listener ("Gtk:AtkObject:children-changed");
   add_signal_listener ("Gtk:AtkObject:visible-data-changed");
@@ -390,15 +424,17 @@ spi_atk_bridge_property_event_listener (GSignalInvocationHint *signal_hint,
 #ifdef SPI_BRIDGE_DEBUG
   GSignalQuery signal_query;
   const gchar *name;
-  gchar *s, *s2;
+  const gchar *s, *s2;
   
   g_signal_query (signal_hint->signal_id, &signal_query);
   name = signal_query.signal_name;
 
   s2 = g_type_name (G_OBJECT_TYPE (g_value_get_object (param_values + 0)));
   s = atk_object_get_name (ATK_OBJECT (g_value_get_object (param_values + 0)));
-  fprintf (stderr, "Received (property) signal %s:%s from object %s (gail %s)\n",
-	   g_type_name (signal_query.itype), name, s, s2);
+  values = (AtkPropertyValues*) g_value_get_pointer (param_values + 1);
+  fprintf (stderr, "Received (property) signal %s:%s:%s from object %s (gail %s)\n",
+	   g_type_name (signal_query.itype), name, values->property_name, s, s2);
+  
 #endif
 
   gobject = g_value_get_object (param_values + 0);
@@ -409,7 +445,6 @@ spi_atk_bridge_property_event_listener (GSignalInvocationHint *signal_hint,
   return TRUE;
 }
 
-#if THIS_WILL_EVER_BE_USED
 static gboolean
 spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
 				     guint n_param_values,
@@ -417,7 +452,9 @@ spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
 				     gpointer data)
 {
   GObject *gobject;
-  AtkPropertyValues *values;
+  gchar *property_name;
+  gchar *type;
+  unsigned long detail1;
 #ifdef SPI_BRIDGE_DEBUG
   GSignalQuery signal_query;
   const gchar *name;
@@ -429,16 +466,19 @@ spi_atk_bridge_state_event_listener (GSignalInvocationHint *signal_hint,
 #endif
 
   gobject = g_value_get_object (param_values + 0);
-  values = (AtkPropertyValues*) g_value_get_pointer (param_values + 1);
-
+  property_name = g_strdup (g_value_get_string (param_values + 1));
+  detail1 = (g_value_get_boolean (param_values + 2))
+    ? 1 : 0;
+  type = g_strdup_printf ("object:state-changed:%s", property_name);
   spi_atk_emit_eventv (gobject, 
-		       (unsigned long) values->old_value.data[0].v_ulong,
-		       (unsigned long) values->new_value.data[0].v_ulong,
-		       "object:%s:?", values->property_name);
-  
+		       detail1,
+		       0,
+		       type);
+  g_free (property_name);
+  g_free (type);
   return TRUE;
 }
-#endif
+
 
 static void
 spi_init_keystroke_from_atk_key_event (Accessibility_DeviceEvent  *keystroke,
@@ -529,7 +569,7 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   const gchar *name;
   gint detail1 = 0, detail2 = 0;
 #ifdef SPI_BRIDGE_DEBUG
-  gchar *s, *s2;
+  const gchar *s, *s2;
 #endif
   
   g_signal_query (signal_hint->signal_id, &signal_query);
@@ -540,7 +580,7 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
   s2 = g_type_name (G_OBJECT_TYPE (g_value_get_object (param_values + 0)));
   s = atk_object_get_name (ATK_OBJECT (g_value_get_object (param_values + 0)));
   fprintf (stderr, "Received signal %s:%s from object %s (gail %s)\n",
-	   g_type_name (signal_query.itype), name, s, s2);
+	   g_type_name (signal_query.itype), name, s ? s : "<NULL>" , s2);
 #endif
 
   gobject = g_value_get_object (param_values + 0);
@@ -553,3 +593,35 @@ spi_atk_bridge_signal_listener (GSignalInvocationHint *signal_hint,
 
   return TRUE;
 }
+
+
+
+static gboolean
+spi_atk_bridge_window_event_listener (GSignalInvocationHint *signal_hint,
+				guint n_param_values,
+				const GValue *param_values,
+				gpointer data)
+{
+  GObject *gobject;
+  GSignalQuery signal_query;
+  const gchar *name;
+#ifdef SPI_BRIDGE_DEBUG
+  const gchar *s, *s2;
+#endif
+  
+  g_signal_query (signal_hint->signal_id, &signal_query);
+
+  name = signal_query.signal_name;
+
+#ifdef SPI_BRIDGE_DEBUG
+  s2 = g_type_name (G_OBJECT_TYPE (g_value_get_object (param_values + 0)));
+  s = atk_object_get_name (ATK_OBJECT (g_value_get_object (param_values + 0)));
+  fprintf (stderr, "Received signal %s:%s from object %s (gail %s)\n",
+	   g_type_name (signal_query.itype), name, s ? s : "<NULL>" , s2);
+#endif
+
+  gobject = g_value_get_object (param_values + 0);
+  spi_atk_emit_eventv (gobject, 0, 0, "window:%s", name);
+
+  return TRUE;
+}
