@@ -1,4 +1,7 @@
-/* ATK -  Accessibility Toolkit
+/*
+ * AT-SPI - Assistive Technology Service Provider Interface
+ * (Gnome Accessibility Project; http://developer.gnome.org/projects/gap)
+ *
  * Copyright 2001 Ximian, Inc.
  *
  * This library is free software; you can redistribute it and/or
@@ -32,6 +35,7 @@
 #include <libspi/image.h>
 #include <libspi/relation.h>
 #include <libspi/selection.h>
+#include <libspi/stateset.h>
 #include <libspi/table.h>
 #include <libspi/text.h>
 #include <libspi/value.h>
